"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var argnames_1 = require("@chengaoyuan/argnames");
var pathRegexp_1 = require("./pathRegexp");
function LazyInitMetaParam(target, methodName) {
    if (!target.$MetaParam) {
        target.$MetaParam = {};
    }
    if (!target.$MetaParam[methodName]) {
        target.$MetaParam[methodName] = [];
    }
    return target.$MetaParam[methodName];
}
function AddMetaParam(target, methodName, paramIndex, paramType, paramName) {
    LazyInitMetaParam(target, methodName)[paramIndex] = {
        name: paramName,
        type: paramType
    };
}
function NormalizeMetaParam(target, methodName, argNames) {
    var params = LazyInitMetaParam(target, methodName);
    for (var index = 0; index < argNames.length; index++) {
        if (!params[index]) {
            params[index] = {
                type: "query",
                name: argNames[index]
            };
        }
        else {
            params[index].name = params[index].name || argNames[index];
        }
    }
    return params;
}
function BuildParamDecorator(type) {
    return function (target, methodName, index) {
        AddMetaParam(target, methodName, index, type, "");
    };
}
function BuildNameParamDecorator(type) {
    function ParamDecorator() {
        if (arguments.length === 3) {
            AddMetaParam(arguments[0], arguments[1], arguments[2], type, "");
        }
        else {
            return function (target, methodName, index) {
                AddMetaParam(target, methodName, index, type, arguments[0]);
            };
        }
    }
    return ParamDecorator;
}
exports.Query = BuildNameParamDecorator("query");
exports.RequestPath = BuildNameParamDecorator("path");
exports.RequestBody = BuildParamDecorator("body");
exports.RequestHeader = BuildNameParamDecorator("header");
exports.RequestHeaders = BuildParamDecorator("headers");
exports.Cookie = BuildNameParamDecorator("cookie");
exports.Cookies = BuildParamDecorator("cookies");
exports.Session = BuildParamDecorator("session");
exports.Request = BuildParamDecorator("request");
exports.Response = BuildParamDecorator("response");
function LazyInitMetaPath(target) {
    if (!target.$MetaPath) {
        target.$MetaPath = [];
    }
    return target.$MetaPath;
}
function AddMetaPath(target, methodName, methodPath, method) {
    LazyInitMetaPath(target).push({ methodName: methodName, methodPath: methodPath, method: method });
}
function RequestMapping(info) {
    return function (target, methodName, des) {
        var argNames = argnames_1.default(des.value);
        NormalizeMetaParam(target, methodName, argNames);
        if (typeof info === "string") {
            AddMetaPath(target, methodName, info, "GET");
        }
        else {
            AddMetaPath(target, methodName, info.path, info.method);
        }
    };
}
exports.RequestMapping = RequestMapping;
var hasOwnProperty = Object.prototype.hasOwnProperty;
function DecodeParam(val) {
    if (typeof val !== "string" || val.length === 0) {
        return val;
    }
    try {
        return decodeURIComponent(val);
    }
    catch (err) {
        if (err instanceof URIError) {
            err.message = "Failed to decode param '" + val + "'";
        }
        throw err;
    }
}
var NullObject = {};
function HandlerRequest(router, req, res, path) {
    var methodParams = router.methodParams;
    var methodParamLength = methodParams.length;
    var args = new Array(methodParamLength);
    var query = req.query;
    var cookies = req.cookies || NullObject;
    var headers = req.headers || NullObject;
    for (var index = 0; index < methodParamLength; index++) {
        var param = methodParams[index];
        switch (param.type) {
            case "path":
                args[index] = path[param.name];
                break;
            case "query":
                {
                    var arg = query[param.name];
                    if (arg === undefined) {
                        res.statusMessage = "Bad Request! Not found param '" + param.name + "'!";
                        res.status(400).end(res.statusMessage);
                        return;
                    }
                    args[index] = arg;
                }
                break;
            case "body":
                args[index] = req.body;
                break;
            case "cookie":
                {
                    var arg = cookies[param.name];
                    if (arg === undefined) {
                        res.statusMessage = "Bad Request! Not found cookie '" + param.name + "'!";
                        res.status(400).end(res.statusMessage);
                        return;
                    }
                    args[index] = arg;
                }
                break;
            case "cookies":
                args[index] = req.cookies;
                break;
            case "header":
                {
                    var arg = headers[param.name];
                    if (arg === undefined) {
                        res.statusMessage = "Bad Request! Not found header '" + param.name + "'!";
                        res.status(400).end(res.statusMessage);
                        return;
                    }
                    args[index] = arg;
                }
                break;
            case "headers":
                args[index] = req.headers;
                break;
            case "session":
                args[index] = req.session;
                break;
            case "request":
                args[index] = req;
                break;
            case "response":
                args[index] = res;
                break;
        }
    }
    var handler = router.target[router.methodName];
    handler
        .apply(router.instance, args)
        .then(function (data) {
        if (!res.finished) {
            res.setHeader("content-type", "application/json;charset=utf-8");
            res.end(JSON.stringify(data));
        }
    })
        .catch(function (err) {
        console.error(err);
        if (!res.finished) {
            res.sendStatus(500).end();
        }
    });
}
var Controller = /** @class */ (function () {
    function Controller() {
        this.routers = [];
    }
    Controller.prototype.handler = function () {
        var _this = this;
        return function (req, res, next) {
            for (var _i = 0, _a = _this.routers; _i < _a.length; _i++) {
                var router = _a[_i];
                if (router.method !== req.method) {
                    continue;
                }
                var match = router.regexp.exec(req.path);
                if (match) {
                    var pathParams = {};
                    var keys = router.keys;
                    for (var i = 1; i < match.length; i++) {
                        var key = keys[i - 1];
                        var prop = key.name;
                        var val = DecodeParam(match[i]);
                        if (val !== undefined || !hasOwnProperty.call(pathParams, prop)) {
                            pathParams[prop] = val;
                        }
                    }
                    HandlerRequest(router, req, res, pathParams);
                    return;
                }
            }
            next();
        };
    };
    Controller.prototype.addController = function (path, cls) {
        var target = cls.prototype;
        var instance = new cls();
        var methodPaths = LazyInitMetaPath(target);
        for (var _i = 0, methodPaths_1 = methodPaths; _i < methodPaths_1.length; _i++) {
            var info = methodPaths_1[_i];
            var methodParams = LazyInitMetaParam(target, info.methodName);
            var realPath = path + info.methodPath;
            var keys = [];
            this.routers.push({
                keys: keys,
                target: target,
                method: info.method,
                regexp: pathRegexp_1.default(realPath, keys),
                instance: instance,
                methodName: info.methodName,
                methodParams: methodParams
            });
        }
    };
    return Controller;
}());
exports.Controller = Controller;
//# sourceMappingURL=index.js.map