"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Match matching groups in a regular expression.
 */
var MATCHING_GROUP_REGEXP = /\((?!\?)/g;
function pathRegExp(path, keys) {
    var options = {};
    var strict = options.strict;
    var end = options.end !== false;
    var flags = options.sensitive ? "" : "i";
    var extraOffset = 0;
    var keysOffset = keys.length;
    var i = 0;
    var name = 0;
    var m;
    path = ("^" + path + (strict ? "" : path[path.length - 1] === "/" ? "?" : "/?"))
        .replace(/\/\(/g, "/(?:")
        .replace(/([\/\.])/g, "\\$1")
        .replace(/(\\\/)?(\\\.)?:(\w+)(\(.*?\))?(\*)?(\?)?/g, function (match, slash, format, key, capture, star, optional, offset) {
        slash = slash || "";
        format = format || "";
        capture = capture || "([^\\/" + format + "]+?)";
        optional = optional || "";
        keys.push({
            name: key,
            optional: !!optional,
            offset: offset + extraOffset
        });
        var result = "" +
            (optional ? "" : slash) +
            "(?:" +
            format +
            (optional ? slash : "") +
            capture +
            (star ? "((?:[\\/" + format + "].+?)?)" : "") +
            ")" +
            optional;
        extraOffset += result.length - match.length;
        return result;
    })
        .replace(/\*/g, function (star, index) {
        var len = keys.length;
        while (len-- > keysOffset && keys[len].offset > index) {
            keys[len].offset += 3; // Replacement length minus asterisk length.
        }
        return "(.*)";
    });
    // This is a workaround for handling unnamed matching groups.
    while ((m = MATCHING_GROUP_REGEXP.exec(path))) {
        var escapeCount = 0;
        var index = m.index;
        while (path.charAt(--index) === "\\") {
            escapeCount++;
        }
        // It's possible to escape the bracket.
        if (escapeCount % 2 === 1) {
            continue;
        }
        if (keysOffset + i === keys.length || keys[keysOffset + i].offset > m.index) {
            keys.splice(keysOffset + i, 0, {
                name: name++,
                optional: false,
                offset: m.index
            });
        }
        i++;
    }
    // If the path is non-ending, match until the end or a slash.
    path += end ? "$" : path[path.length - 1] === "/" ? "" : "(?=\\/|$)";
    return new RegExp(path, flags);
}
exports.default = pathRegExp;
//# sourceMappingURL=pathRegexp.js.map